"""Scraper for Supreme Court of U.S.
CourtID: scotus
Court Short Name: scotus
History:
 - 2014-07-20 - Created by Andrei Chelaru, reviewed by MLR
 - 2017-10-09 - Updated by MLR.
"""

from datetime import datetime

from juriscraper.OralArgumentSite import OralArgumentSite


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = 'http://www.supremecourt.gov/oral_arguments/argument_audio.aspx'
        self.back_scrape_iterable = range(2010, 2015)

    def _get_download_urls(self):
        path = "id('list')//tr//a/text()"
        return map(self._return_download_url, self.html.xpath(path))

    @staticmethod
    def _return_download_url(d):
        file_type = 'mp3' # or 'wma' is also available for any case.
        download_url = 'http://www.supremecourt.gov/media/audio/{type}files/{docket_number}.{type}'.format(
            type=file_type,
            docket_number=d
        )
        return download_url

    def _get_case_names(self):
        path = "id('list')//tr/td/span/text()"
        return [s.lstrip('. ') for s in self.html.xpath(path)]

    def _get_case_dates(self):
        path = "id('list')//tr/td[2]//text()"
        return [datetime.strptime(s, '%m/%d/%y').date() for s in
                self.html.xpath(path) if not 'Date' in s]

    def _get_docket_numbers(self):
        path = "id('list')//tr//a/text()"
        return list(self.html.xpath(path))

    def _download_backwards(self, year):
        self.url = 'http://www.supremecourt.gov/oral_arguments/argument_audio/%s' % year
        self.html = self._download()



"""Scraper for Tenth Circuit of Appeals
CourtID: ca10
Author: mlr
Date created: 6 Sept. 2018
"""
import re

from juriscraper.OralArgumentSite import OralArgumentSite
from juriscraper.lib.string_utils import convert_date_string


class Site(OralArgumentSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.url = 'https://www.ca10.uscourts.gov/clerk/oral-argument-recordings'
        self.back_scrape_iterable = [1, 2, 3]
        self._anchor_nodes = None

    def _get_anchor_nodes(self):
        if self._anchor_nodes:
            return self._anchor_nodes

        all_anchors = self.html.xpath('//a')
        mp3_anchors = []
        for anchor in all_anchors:
            hrefs = anchor.xpath('./@href')
            if hrefs and hrefs[0].lower().endswith('.mp3'):
                mp3_anchors.append(anchor)

        # Cache and return it.
        self._anchor_nodes = mp3_anchors
        return mp3_anchors

    def _get_download_urls(self):
        nodes = self._get_anchor_nodes()
        return [node.xpath('./@href')[0] for node in nodes]

    @staticmethod
    def __split_anchor_text(s):
        """Split the anchor text into the docket number and the case name

        Examples: [
            '17-2085, United States v. Roach, Appellant',
            '17-7042 & 17-7044, The Cherokee Nation v. Zinke, et al., Appellants',
            '17-2027, 17-2035, United States v. Solis, Appellant',
        ]

        :return tuple of docket numbers & case name
        """
        # Make the docket number optional. Sometimes it's missing.
        regex = re.compile('(?:(.*\d\d-\d{1,4}), )?(.*)')
        results = regex.search(s)
        docket_number = results.group(1)
        case_name = results.group(2)
        if not docket_number:
            docket_number = "Unknown"
        return docket_number.strip(), case_name.strip()

    def _get_case_names(self):
        nodes = self._get_anchor_nodes()
        case_names = []
        for node in nodes:
            text = node.text_content()
            case_names.append(self.__split_anchor_text(text)[1])
        return case_names

    def _get_docket_numbers(self):
        nodes = self._get_anchor_nodes()
        docket_numbers = []
        for node in nodes:
            text = node.text_content()
            docket_numbers.append(self.__split_anchor_text(text)[0])
        return docket_numbers

    def _get_case_dates(self):
        nodes = self._get_anchor_nodes()
        dates = []
        for node in nodes:
            d_str = node.xpath('./preceding::h3')[0].text_content()
            dates.append(convert_date_string(d_str, fuzzy=True))
        return dates

    def _download_backwards(self, i):
        self.url = 'https://www.ca10.uscourts.gov/clerk/oral-argument-recordings?page=%s' % i
        self.html = self._download()



# -*- coding: utf-8 -*-

# Scraper for Pennsylvania Supreme Court
# CourtID: pa
# Court Short Name: pa
# Author: Andrei Chelaru
# Reviewer: mlr
# Date created: 21 July 2014

import re

from juriscraper.OpinionSite import OpinionSite
from juriscraper.lib.string_utils import convert_date_string
from juriscraper.lib.string_utils import clean_string


class Site(OpinionSite):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.regex = False
        self.url = 'http://www.pacourts.us/assets/rss/SupremeOpinionsRss.ashx'
        self.set_regex(r"(.*)(?:[,-]?\s+Nos?\.)(.*)")
        self.base = "//item[not(contains(title/text(), 'Judgment List'))]" \
                    "[not(contains(title/text(), 'Reargument Table'))]" \
                    "[contains(title/text(), 'No.')]"

    def set_regex(self, pattern):
        self.regex = re.compile(pattern)

    def _get_case_names(self):
        path = "%s/title" % self.base
        return [self.return_case_name(s) for s in self.return_path_text(path)]

    def _get_download_urls(self):
        path = '%s//@href' % self.base
        return [href for href in self.html.xpath(path)]

    def _get_case_dates(self):
        path = "%s/pubdate" % self.base
        return [convert_date_string(s) for s in self.return_path_text(path)]

    def _get_precedential_statuses(self):
        return ['Published'] * len(self.case_names)

    def _get_docket_numbers(self):
        path = "%s/title" % self.base
        return [self.return_docket_number(s) for s in self.return_path_text(path)]

    def _get_judges(self):
        return self.return_path_text('%s/creator' % self.base)

    def return_path_text(self, path):
        return [self.sanitize_text(element.text_content()) for element in self.html.xpath(path)]

    def return_case_name(self, text):
        text = clean_string(text)
        return self.regex.search(text).group(1)

    def return_docket_number(self, text):
        text = clean_string(text)
        return self.regex.search(text).group(2)

    def sanitize_text(self, text):
        text = clean_string(text)
        return text.replace(r'\n', '\n').replace(u'–', '-')
